@@ -156,6 +156,9 @@ class DummyChannel(InstrumentChannel):
         self.add_parameter(name='dummy_2d_multi_parameter',
                            parameter_class=Multi2DSetPointParam)
 
+        self.add_parameter(name='dummy_2d_multi_parameter_2',
+                           parameter_class=Multi2DSetPointParam2Sizes)
+
         self.add_parameter(name='dummy_array_parameter',
                            parameter_class=ArraySetPointParam)
 
@@ -271,14 +274,14 @@ class MultiSetPointParam(MultiParameter):
     Multiparameter which only purpose it to test that units, setpoints
     and so on are copied correctly to the individual arrays in the datarray.
     """
-    def __init__(self, instrument=None, name='testparameter'):
+    def __init__(self, instrument=None, name='multi_setpoint_param'):
         shapes = ((5,), (5,))
-        names = ('this', 'that')
+        names = ('multi_setpoint_param_this', 'multi_setpoint_param_that')
         labels = ('this label', 'that label')
         units = ('this unit', 'that unit')
         sp_base = tuple(np.linspace(5, 9, 5))
         setpoints = ((sp_base,), (sp_base,))
-        setpoint_names = (('this_setpoint',), ('this_setpoint',))
+        setpoint_names = (('multi_setpoint_param_this_setpoint',), ('multi_setpoint_param_this_setpoint',))
         setpoint_labels = (('this setpoint',), ('this setpoint',))
         setpoint_units = (('this setpointunit',), ('this setpointunit',))
         super().__init__(name, names, shapes,
@@ -301,7 +304,7 @@ class Multi2DSetPointParam(MultiParameter):
     and so on are copied correctly to the individual arrays in the datarray.
     """
 
-    def __init__(self, instrument=None, name='testparameter'):
+    def __init__(self, instrument=None, name='multi_2d_setpoint_param'):
         shapes = ((5, 3), (5, 3))
         names = ('this', 'that')
         labels = ('this label', 'that label')
@@ -310,8 +313,8 @@ class Multi2DSetPointParam(MultiParameter):
         sp_base_2 = tuple(np.linspace(9, 11, 3))
         array_setpoints = setpoint_generator(sp_base_1, sp_base_2)
         setpoints = (array_setpoints, array_setpoints)
-        setpoint_names = (('this_setpoint', 'that_setpoint'),
-                          ('this_setpoint', 'that_setpoint'))
+        setpoint_names = (('multi_2d_setpoint_param_this_setpoint', 'multi_2d_setpoint_param_that_setpoint'),
+                          ('multi_2d_setpoint_param_this_setpoint', 'multi_2d_setpoint_param_that_setpoint'))
         setpoint_labels = (('this setpoint', 'that setpoint'),
                            ('this setpoint', 'that setpoint'))
         setpoint_units = (('this setpointunit',
@@ -332,6 +335,47 @@ class Multi2DSetPointParam(MultiParameter):
         return items
 
 
+
+class Multi2DSetPointParam2Sizes(MultiParameter):
+    """
+    Multiparameter for testing containing individual parameters with different
+    shapes.
+    """
+
+    def __init__(self, instrument=None, name='multi_2d_setpoint_param'):
+        shapes = ((5, 3), (2, 7))
+        names = ('this_5_3', 'this_2_7')
+        labels = ('this label', 'that label')
+        units = ('this unit', 'that unit')
+        sp_base_1_1 = tuple(np.linspace(5, 9, 5))
+        sp_base_2_1 = tuple(np.linspace(9, 11, 3))
+        array_setpoints_1 = setpoint_generator(sp_base_1_1, sp_base_2_1)
+        sp_base_1_2 = tuple(np.linspace(5, 9, 2))
+        sp_base_2_2 = tuple(np.linspace(9, 11, 7))
+        array_setpoints_2 = setpoint_generator(sp_base_1_2, sp_base_2_2)
+        setpoints = (array_setpoints_1, array_setpoints_2)
+        setpoint_names = (('multi_2d_setpoint_param_this_setpoint_1', 'multi_2d_setpoint_param_that_setpoint_1'),
+                          ('multi_2d_setpoint_param_this_setpoint_2', 'multi_2d_setpoint_param_that_setpoint_2'))
+        setpoint_labels = (('this setpoint 1', 'that setpoint 1'),
+                           ('this setpoint 2', 'that setpoint 2'))
+        setpoint_units = (('this setpointunit',
+                           'that setpointunit'),
+                          ('this setpointunit',
+                           'that setpointunit'))
+        super().__init__(name, names, shapes,
+                         instrument=instrument,
+                         labels=labels,
+                         units=units,
+                         setpoints=setpoints,
+                         setpoint_labels=setpoint_labels,
+                         setpoint_names=setpoint_names,
+                         setpoint_units=setpoint_units)
+
+    def get_raw(self):
+        items = (np.zeros((5, 3)), np.ones((2, 7)))
+        return items
+
+
 class MultiScalarParam(MultiParameter):
     """
     Multiparameter whos elements are scalars i.e. similar to
@@ -360,13 +404,13 @@ class ArraySetPointParam(ArrayParameter):
     and so on are copied correctly to the individual arrays in the datarray.
     """
 
-    def __init__(self, instrument=None, name='testparameter'):
+    def __init__(self, instrument=None, name='array_setpoint_param'):
         shape = (5,)
         label = 'this label'
         unit = 'this unit'
         sp_base = tuple(np.linspace(5, 9, 5))
         setpoints = (sp_base,)
-        setpoint_names = ('this_setpoint',)
+        setpoint_names = ('array_setpoint_param_this_setpoint',)
         setpoint_labels = ('this setpoint',)
         setpoint_units = ('this setpointunit',)
         super().__init__(name,
@@ -384,7 +428,6 @@ class ArraySetPointParam(ArrayParameter):
         return item
 
 
-
 class ComplexArraySetPointParam(ArrayParameter):
     """
     Arrayparameter that returns complex numbers
@@ -510,11 +553,11 @@ class SnapShotTestInstrument(Instrument):
                                get_cmd=partial(self._getter, p_name))
 
     def _getter(self, name: str):
-        val = self.parameters[name].cache._value
+        val = self.parameters[name].cache.get(get_if_invalid=False)
         self._get_calls[name] += 1
         return val
 
-    def snapshot_base(self, update: bool = True,
+    def snapshot_base(self, update: Optional[bool] = True,
                       params_to_skip_update: Optional[Sequence[str]] = None
                       ) -> Dict:
         if params_to_skip_update is None:
