@@ -332,63 +332,6 @@ def wait_secs(finish_clock):
     return delay
 
 
-class LogCapture():
-
-    """
-    Context manager to grab all log messages, optionally
-    from a specific logger.
-
-    usage::
-
-        with LogCapture() as logs:
-            code_that_makes_logs(...)
-        log_str = logs.value
-
-    """
-
-    @deprecate(reason="The logging infrastructure has moved to `qcodes.utils.logger`",
-               alternative="`qcodes.utils.logger.LogCapture`")
-    def __init__(self, logger=logging.getLogger()):
-        self.logger = logger
-
-        self.stashed_handlers = self.logger.handlers[:]
-        for handler in self.stashed_handlers:
-            self.logger.removeHandler(handler)
-
-    def __enter__(self):
-        self.log_capture = io.StringIO()
-        self.string_handler = logging.StreamHandler(self.log_capture)
-        self.string_handler.setLevel(logging.DEBUG)
-        self.logger.addHandler(self.string_handler)
-        return self
-
-    def __exit__(self, type, value, tb):
-        self.logger.removeHandler(self.string_handler)
-        self.value = self.log_capture.getvalue()
-        self.log_capture.close()
-
-        for handler in self.stashed_handlers:
-            self.logger.addHandler(handler)
-
-
-@deprecate(
-    reason="This method is no longer being used in QCoDeS.")
-def make_unique(s, existing):
-    """
-    Make string ``s`` unique, able to be added to a sequence ``existing`` of
-    existing names without duplication, by ``appending _<int>`` to it if needed.
-    """
-    n = 1
-    s_out = s
-    existing = set(existing)
-
-    while s_out in existing:
-        n += 1
-        s_out = '{}_{}'.format(s, n)
-
-    return s_out
-
-
 class DelegateAttributes:
     """
     Mixin class to create attributes of this object by
@@ -626,7 +569,7 @@ def add_to_spyder_UMR_excludelist(modulename: str):
             except ImportError:
                 pass
             else:
-                print("found kernels site")
+                logging.info("found spyder kernels site")
                 sitecustomize_found = True
 
         if sitecustomize_found is False:
